]> sigrok.org Git - libsigrok.git/blobdiff - device.c
Doxygen: Add @since markers.
[libsigrok.git] / device.c
index 84d3d663dd6ff459589d844cd70cd40cd6776587..62395be57f3de405f6522e327d458d74f345d8c4 100644 (file)
--- a/device.c
+++ b/device.c
@@ -1,7 +1,7 @@
 /*
  * This file is part of the sigrok project.
  *
- * Copyright (C) 2010-2012 Bert Vermeulen <bert@biot.com>
+ * Copyright (C) 2013 Bert Vermeulen <bert@biot.com>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
 
 #include <stdio.h>
 #include <glib.h>
+#include "config.h" /* Needed for HAVE_LIBUSB_1_0 and others. */
 #include "libsigrok.h"
 #include "libsigrok-internal.h"
 
+/* Message logging helpers with driver-specific prefix string. */
+#define DRIVER_LOG_DOMAIN "device: "
+#define sr_log(l, s, args...) sr_log(l, DRIVER_LOG_DOMAIN s, ## args)
+#define sr_spew(s, args...) sr_spew(DRIVER_LOG_DOMAIN s, ## args)
+#define sr_dbg(s, args...) sr_dbg(DRIVER_LOG_DOMAIN s, ## args)
+#define sr_info(s, args...) sr_info(DRIVER_LOG_DOMAIN s, ## args)
+#define sr_warn(s, args...) sr_warn(DRIVER_LOG_DOMAIN s, ## args)
+#define sr_err(s, args...) sr_err(DRIVER_LOG_DOMAIN s, ## args)
+
+/**
+ * @file
+ *
+ * Device handling in libsigrok.
+ */
+
+/**
+ * @defgroup grp_devices Devices
+ *
+ * Device handling in libsigrok.
+ *
+ * @{
+ */
+
+/** @private */
 SR_PRIV struct sr_probe *sr_probe_new(int index, int type,
                gboolean enabled, const char *name)
 {
        struct sr_probe *probe;
 
        if (!(probe = g_try_malloc0(sizeof(struct sr_probe)))) {
-               sr_err("hwdriver: probe malloc failed");
+               sr_err("Probe malloc failed.");
                return NULL;
        }
 
@@ -54,6 +79,8 @@ SR_PRIV struct sr_probe *sr_probe_new(int index, int type,
  *             of the string is made.
  *
  * @return SR_OK on success, or SR_ERR_ARG on invalid arguments.
+ *
+ * @since 0.1.0 (but the API changed in 0.2.0)
  */
 SR_API int sr_dev_probe_name_set(const struct sr_dev_inst *sdi,
                int probenum, const char *name)
@@ -89,6 +116,8 @@ SR_API int sr_dev_probe_name_set(const struct sr_dev_inst *sdi,
  * @param state TRUE to enable the probe, FALSE to disable.
  *
  * @return SR_OK on success, or SR_ERR_ARG on invalid arguments.
+ *
+ * @since 0.2.0
  */
 SR_API int sr_dev_probe_enable(const struct sr_dev_inst *sdi, int probenum,
                gboolean state)
@@ -124,6 +153,8 @@ SR_API int sr_dev_probe_enable(const struct sr_dev_inst *sdi, int probenum,
  * @param trigger Trigger string, in the format used by sigrok-cli
  *
  * @return SR_OK on success, or SR_ERR_ARG on invalid arguments.
+ *
+ * @since 0.1.0 (but the API changed in 0.2.0)
  */
 SR_API int sr_dev_trigger_set(const struct sr_dev_inst *sdi, int probenum,
                const char *trigger)
@@ -151,46 +182,56 @@ SR_API int sr_dev_trigger_set(const struct sr_dev_inst *sdi, int probenum,
 }
 
 /**
* Determine whether the specified device has the specified capability.
  *
- * @param dev Pointer to the device instance to be checked. Must not be NULL.
+ * @param sdi Pointer to the device instance to be checked. Must not be NULL.
  *            If the device's 'driver' field is NULL (virtual device), this
  *            function will always return FALSE (virtual devices don't have
  *            a hardware capabilities list).
* @param hwcap The capability that should be checked (whether it's supported
*              by the specified device).
  *
- * @return TRUE if the device has the specified capability, FALSE otherwise.
- *         FALSE is also returned upon invalid input parameters or other
+ * @return TRUE if the device has the specified option, FALSE otherwise.
+ *         FALSE is also returned on invalid input parameters or other
  *         error conditions.
+ *
+ * @since 0.1.0 (but the API changed in 0.2.0)
  */
-SR_API gboolean sr_dev_has_hwcap(const struct sr_dev_inst *sdi, int hwcap)
+SR_API gboolean sr_dev_has_option(const struct sr_dev_inst *sdi, int key)
 {
-       const int *hwcaps;
-       int i;
+       GVariant *gvar;
+       const int *devopts;
+       gsize num_opts, i;
+       int ret;
 
-       if (!sdi || !sdi->driver)
+       if (!sdi || !sdi->driver || !sdi->driver->config_list)
                return FALSE;
 
-       if (sdi->driver->info_get(SR_DI_HWCAPS,
-                       (const void **)&hwcaps, NULL) != SR_OK)
+       if (sdi->driver->config_list(SR_CONF_DEVICE_OPTIONS, &gvar, NULL) != SR_OK)
                return FALSE;
 
-       for (i = 0; hwcaps[i]; i++) {
-               if (hwcaps[i] == hwcap)
-                       return TRUE;
+       ret = FALSE;
+       devopts = g_variant_get_fixed_array(gvar, &num_opts, sizeof(int32_t));
+       for (i = 0; i < num_opts; i++) {
+               if (devopts[i] == key) {
+                       ret = TRUE;
+                       break;
+               }
        }
+       g_variant_unref(gvar);
 
-       return FALSE;
+       return ret;
 }
 
+/** @private */
 SR_PRIV struct sr_dev_inst *sr_dev_inst_new(int index, int status,
                const char *vendor, const char *model, const char *version)
 {
        struct sr_dev_inst *sdi;
 
        if (!(sdi = g_try_malloc(sizeof(struct sr_dev_inst)))) {
-               sr_err("hwdriver: %s: sdi malloc failed", __func__);
+               sr_err("Device instance malloc failed.");
                return NULL;
        }
 
@@ -207,6 +248,7 @@ SR_PRIV struct sr_dev_inst *sr_dev_inst_new(int index, int status,
        return sdi;
 }
 
+/** @private */
 SR_PRIV void sr_dev_inst_free(struct sr_dev_inst *sdi)
 {
        struct sr_probe *probe;
@@ -223,18 +265,18 @@ SR_PRIV void sr_dev_inst_free(struct sr_dev_inst *sdi)
        g_free(sdi->model);
        g_free(sdi->version);
        g_free(sdi);
-
 }
 
 #ifdef HAVE_LIBUSB_1_0
 
+/** @private */
 SR_PRIV struct sr_usb_dev_inst *sr_usb_dev_inst_new(uint8_t bus,
                        uint8_t address, struct libusb_device_handle *hdl)
 {
        struct sr_usb_dev_inst *udi;
 
        if (!(udi = g_try_malloc(sizeof(struct sr_usb_dev_inst)))) {
-               sr_err("hwdriver: %s: udi malloc failed", __func__);
+               sr_err("USB device instance malloc failed.");
                return NULL;
        }
 
@@ -245,9 +287,9 @@ SR_PRIV struct sr_usb_dev_inst *sr_usb_dev_inst_new(uint8_t bus,
        return udi;
 }
 
+/** @private */
 SR_PRIV void sr_usb_dev_inst_free(struct sr_usb_dev_inst *usb)
 {
-       /* Avoid compiler warnings. */
        (void)usb;
 
        /* Nothing to do for this device instance type. */
@@ -255,45 +297,55 @@ SR_PRIV void sr_usb_dev_inst_free(struct sr_usb_dev_inst *usb)
 
 #endif
 
+/**
+ * @private
+ *
+ * Both parameters are copied to newly allocated strings, and freed
+ * automatically by sr_serial_dev_inst_free().
+ *
+ * @param pathname OS-specific serial port specification. Examples:
+ *                 "/dev/ttyUSB0", "/dev/ttyACM1", "/dev/tty.Modem-0", "COM1".
+ * @param serialcomm A serial communication parameters string, in the form
+ *                   of <speed>/<data bits><parity><stopbits>, for example
+ *                   "9600/8n1" or "600/7o2". This is an optional parameter;
+ *                   it may be filled in later.
+ *
+ * @return A pointer to a newly initialized struct sr_serial_dev_inst,
+ *         or NULL on error.
+ */
 SR_PRIV struct sr_serial_dev_inst *sr_serial_dev_inst_new(const char *port,
-                                                         int fd)
+               const char *serialcomm)
 {
        struct sr_serial_dev_inst *serial;
 
-       if (!(serial = g_try_malloc(sizeof(struct sr_serial_dev_inst)))) {
-               sr_err("hwdriver: %s: serial malloc failed", __func__);
+       if (!port) {
+               sr_err("Serial port required.");
+               return NULL;
+       }
+
+       if (!(serial = g_try_malloc0(sizeof(struct sr_serial_dev_inst)))) {
+               sr_err("Serial device instance malloc failed.");
                return NULL;
        }
 
        serial->port = g_strdup(port);
-       serial->fd = fd;
+       if (serialcomm)
+               serial->serialcomm = g_strdup(serialcomm);
+       serial->fd = -1;
 
        return serial;
 }
 
+/** @private */
 SR_PRIV void sr_serial_dev_inst_free(struct sr_serial_dev_inst *serial)
 {
        g_free(serial->port);
-}
-
-SR_API int sr_dev_config_set(const struct sr_dev_inst *sdi, int hwcap,
-               const void *value)
-{
-       int ret;
-
-       if (!sdi || !sdi->driver || !sdi->driver->dev_config_set) {
-               sr_err("hwdriver: unable to set config option");
-               return SR_ERR;
-       }
-
-       ret = sdi->driver->dev_config_set(sdi, hwcap, value);
-
-       return ret;
+       g_free(serial->serialcomm);
+       g_free(serial);
 }
 
 SR_API GSList *sr_dev_inst_list(const struct sr_dev_driver *driver)
 {
-
        if (driver && driver->dev_list)
                return driver->dev_list();
        else
@@ -302,10 +354,10 @@ SR_API GSList *sr_dev_inst_list(const struct sr_dev_driver *driver)
 
 SR_API int sr_dev_inst_clear(const struct sr_dev_driver *driver)
 {
-
        if (driver && driver->dev_clear)
                return driver->dev_clear();
        else
                return SR_OK;
 }
 
+/** @} */
@@ -151,46 +182,56 @@ SR_API int sr_dev_trigger_set(const struct sr_dev_inst *sdi, int probenum,
 
 /**
- * Determine whether the specified device has the specified capability.
+ * Determine whether the specified device instance has the specified
+ * capability.
  *
@@ -151,46 +182,56 @@ SR_API int sr_dev_trigger_set(const struct sr_dev_inst *sdi, int probenum,
  *            function will always return FALSE (virtual devices don't have
  *            a hardware capabilities list).
- * @param hwcap The capability that should be checked (whether it's supported
- *              by the specified device).
+ * @param key The option that should be checked for support on the
+ *            specified device.
  *